# The complet name of the package
# the path MUST be valid or the package won't be loaded
package Ocsinventory::Agent::Backend::OS::Linux::CernExtension;

use strict;
use Data::UUID;

# I need to declare $runAfter because of the strict mode
use vars qw($runAfter);

# The package must be run after OS::Generic
$runAfter = ["Ocsinventory::Agent::Backend::OS::Generic"];

# This is the check function. The agent runs it just once the module is loaded.
# If the function return false, the module and its children are not executed
# eg: OS::Linux and OS::Linux::* won't executed if this run() function return
# false
# It's a easy way to target a module just for a system like Linux in this case.
sub check { $^O =~ /^linux$/ }

# trim string whitespaces function
sub trim {
	my $string = shift;
	$string =~ s/^\s+//;
	$string =~ s/\s+$//;
	return $string;
}

# helper for MySQL date format
sub formatMySQL {
	my $param  = shift;
	my $result = "";
	if ( length($param) == 1 ) {
		$result = "0" . $param;
	}
	else {
		$result = $param;
	}
	return $result;
}

# main function which is executed by OCS
sub run {
	my $params     = shift;                          # I fetch the parameter
	my $logger = $params->{logger};
	my $config = $params->{config};
	my $idFileName = $config->{config}{vardir}."/id.conf";    # system id file

	my $systemId       = "";
	my $systemArch     = "";
	my $systemBootTime = "";

	# test if id is already generated
	if ( -e $idFileName ) {
		# load the id
		open( IDFILE, "$idFileName" ) || do {
			$logger->error("Could not open file $idFileName for writing: $!");
			return ;
		};
		$systemId = <IDFILE>;
		close(IDFILE);
	}

	# if id cannot be read or not generated then generate a new one
	if ( trim($systemId) eq "" ) {
		my $ug = new Data::UUID;
		$systemId = $ug->create_str();

		open( IDFILE, ">$idFileName" ) || do {
			$logger->error("Could not open file $idFileName for writing: $!");
                        return ;
                };
		print IDFILE ($systemId);
		close(IDFILE);
	}

	# get the architecture
	$systemArch = trim(`uname -m`);

	# get the boot time
	my $input = "/proc/uptime";
	open( INPUT, "$input" ) || do {
		$logger->error("Could not open file $input: $!\n");
		return ;
	};
	my $secsago;
	while (<INPUT>) {
		($secsago) = ( split(/\s+/) )[0];
	}
	my $boottime = time() - $secsago;

	( my $sec, my $min, my $hour, my $mday, my $mon, my $year ) =
	  localtime($boottime);
	$year += 1900;
	$systemBootTime = $year . "-"
	  . formatMySQL( $mon + 1 ) . "-"
	  . formatMySQL($mday) . " "
	  . formatMySQL($hour) . ":"
	  . formatMySQL($min) . ":"
	  . formatMySQL($sec);

	# I need to get the inventory object to update it
	my $inventory = $params->{inventory};

	# and I updated the information collected
	$inventory->setCernExtension(
		{
			OS_ID        => $systemId,
			ARCHITECTURE => $systemArch,
			BOOT_TIME    => $systemBootTime
		}
	);
}

1;
